export { default as compile } from './compile/index.js';
export { default as parse } from './parse/index.js';
export { default as preprocess } from './preprocess/index.js';
export { walk } from 'estree-walker';
export { VERSION } from '../shared/version.js';
