import type ts from 'typescript/lib/tsserverlibrary';
import { Logger } from '../logger';
export declare function decorateDiagnostics(ls: ts.LanguageService, info: ts.server.PluginCreateInfo, typescript: typeof ts, logger: Logger): void;
