import type ts from 'typescript/lib/tsserverlibrary';
export declare class Logger {
    private tsLogService;
    private logDebug;
    constructor(tsLogService: ts.server.Logger, suppressNonSvelteLogs?: boolean, logDebug?: boolean);
    log(...args: any[]): void;
    debug(...args: any[]): void;
}
