import type ts from 'typescript';
import { SvelteSnapshotManager } from '../svelte-snapshots';
type _ts = typeof ts;
export declare function decorateQuickFixAndRefactor(ls: ts.LanguageService, ts: _ts, snapshotManager: SvelteSnapshotManager): void;
export {};
