import type ts from 'typescript/lib/tsserverlibrary';
import { SvelteSnapshotManager } from '../svelte-snapshots';
export declare function decorateFileReferences(ls: ts.LanguageService, snapshotManager: SvelteSnapshotManager): void;
