import type ts from 'typescript/lib/tsserverlibrary';
import { SvelteSnapshotManager } from '../svelte-snapshots';
export declare function decorateCallHierarchy(ls: ts.LanguageService, snapshotManager: SvelteSnapshotManager, typescript: typeof ts): void;
