import type ts from 'typescript/lib/tsserverlibrary';
import { Logger } from './logger';
type _ts = typeof ts;
/**
 * This should only be accessed by TS svelte module resolution.
 */
export declare function createSvelteSys(ts: _ts, logger: Logger): ts.System;
export {};
