import type ts from 'typescript/lib/tsserverlibrary';
type LineChar = ts.LineAndCharacter;
type FileMapping = LineMapping[];
type LineMapping = CharacterMapping[];
type CharacterMapping = [
    number,
    number,
    number,
    number
];
export declare class SourceMapper {
    private mappings;
    private reverseMappings?;
    constructor(mappings: FileMapping | string);
    getOriginalPosition(position: LineChar): LineChar;
    getGeneratedPosition(position: LineChar): LineChar;
    private computeReversed;
}
export {};
