export interface Configuration {
    global?: boolean;
    enable: boolean;
    /** Skip the Svelte detection and assume this is a Svelte project */
    assumeIsSvelteProject: boolean;
}
export declare class ConfigManager {
    private emitter;
    private config;
    onConfigurationChanged(listener: (config: Configuration) => void): void;
    removeConfigurationChangeListener(listener: (config: Configuration) => void): void;
    isConfigChanged(config: Configuration): boolean;
    updateConfigFromPluginConfig(config: Configuration): void;
    getConfig(): Configuration;
}
