import type ts from 'typescript/lib/tsserverlibrary';
import { Logger } from '../logger';
type _ts = typeof ts;
export declare function decorateInlayHints(ls: ts.LanguageService, info: ts.server.PluginCreateInfo, ts: _ts, logger: Logger): void;
export {};
