import type ts from 'typescript/lib/tsserverlibrary';
import { Logger } from '../logger';
import { SvelteSnapshotManager } from '../svelte-snapshots';
export declare function decorateUpdateImports(ls: ts.LanguageService, snapshotManager: SvelteSnapshotManager, logger: Logger): void;
